# <%= project_name %>

As of <%= Time.now.strftime("%B %e, %Y %l:%M%P") %>. <%= dependencies.size %> total

## Summary
<% grouped_dependencies.each do |license_name, group| -%>
* <%= group.size %> <%= license_name %>
<% end %>

<% if unapproved_dependencies.any? -%>
## Action
<%= unapproved_dependencies.size %> *unapproved*

<% unapproved_dependencies.each do |dependency| -%>
* <%= link_to dependency.name %> (<%= license_names(dependency) %>)
<% end %>
<% end -%>

## Items

<% sorted_dependencies.each do |dependency| -%>

<a name="<%= dependency.name %>"></a>
### <%= link_to_dependency(dependency) %> <%= version_groups(dependency) %>
#### <%= dependency.summary %>

<% if dependency.is_a?(MergedPackage) && !dependency.aggregate_paths.nil? && dependency.aggregate_paths.count > 0 -%>
##### Paths
<% dependency.aggregate_paths.each do |path| -%>
* <%= path %>
<% end -%>
<% end -%>

<% if dependency.approved_manually? -%>
<%= license_links(dependency) %> manually approved

><%= dependency.manual_approval.why %>

><cite> <%= dependency.manual_approval.who %> <%= dependency.manual_approval.safe_when.to_date %></cite>
<% elsif dependency.permitted? -%>
<%= license_links(dependency) %> permitted
<% else -%>
<%= license_links(dependency) %> _**unapproved**_
<% end -%>

<% if dependency.description -%>
<%= dependency.description %>
<% end -%>
<% end -%>
